// Matrix.c
//
//
//  Author:	Brett Porter
//	Email: brettporter@yahoo.com
//	Website: http://rsn.gamedev.net/pl3d
//	Copyright (C)2000, 2001, Brett Porter. All Rights Reserved.
//	This source code is released under the LGPL. See license.txt for details.

//	Created: 9 December 2000
//	Last Edited: 8 January 2001
//

#include "math.h"
#include "Matrix.h"

#define PI	3.14159268

float * Matrix_GetMatrix(MATRIX * pM)
{
	return &pM->matrix[0];
}

void Matrix_Matrix(MATRIX * pM)
{
	Matrix_LoadIdentity(pM);
}

void Matrix_PostMultiply(MATRIX * pM, const MATRIX * pmatrix )
{
	float newMatrix[16];
	const float  * m1 = pM->matrix, * m2 = Matrix_GetMatrix(pmatrix); //pmatrix->matrix;

	newMatrix[0] = m1[0] * m2[0] + m1[4] * m2[1] + m1[8] * m2[2];
	newMatrix[1] = m1[1] * m2[0] + m1[5] * m2[1] + m1[9] * m2[2];
	newMatrix[2] = m1[2] * m2[0] + m1[6] * m2[1] + m1[10] * m2[2];
	newMatrix[3] = 0;

	newMatrix[4] = m1[0] * m2[4] + m1[4] * m2[5] + m1[8] * m2[6];
	newMatrix[5] = m1[1] * m2[4] + m1[5] * m2[5] + m1[9] * m2[6];
	newMatrix[6] = m1[2] * m2[4] + m1[6] * m2[5] + m1[10] * m2[6];
	newMatrix[7] = 0;

	newMatrix[8] = m1[0] * m2[8] + m1[4] * m2[9] + m1[8] * m2[10];
	newMatrix[9] = m1[1] * m2[8] + m1[5] * m2[9] + m1[9] * m2[10];
	newMatrix[10] = m1[2] * m2[8] + m1[6] * m2[9] + m1[10] * m2[10];
	newMatrix[11] = 0;

	newMatrix[12] = m1[0] * m2[12] + m1[4] * m2[13] + m1[8] * m2[14] + m1[12];
	newMatrix[13] = m1[1] * m2[12] + m1[5] * m2[13] + m1[9] * m2[14] + m1[13];
	newMatrix[14] = m1[2] * m2[12] + m1[6] * m2[13] + m1[10] * m2[14] + m1[14];
	newMatrix[15] = 1;

	Matrix_Set(pM, newMatrix );
}

void Matrix_SetTranslation(MATRIX * pM, const float *translation )
{
	pM->matrix[12] = translation[0];
	pM->matrix[13] = translation[1];
	pM->matrix[14] = translation[2];
}

void Matrix_SetInverseTranslation(MATRIX * pM, const float *translation )
{
	pM->matrix[12] = -translation[0];
	pM->matrix[13] = -translation[1];
	pM->matrix[14] = -translation[2];
}

void Matrix_SetRotationDegrees(MATRIX * pM, const float *angles )
{
	float vec[3];
	vec[0] = ( float )( angles[0]*180.0/PI );
	vec[1] = ( float )( angles[1]*180.0/PI );
	vec[2] = ( float )( angles[2]*180.0/PI );
	Matrix_SetRotationRadians(pM, vec );
}

void Matrix_SetInverseRotationRadians(MATRIX * pM, const float * angles )
{
	double cr = cos( angles[0] );
	double sr = sin( angles[0] );
	double cp = cos( angles[1] );
	double sp = sin( angles[1] );
	double cy = cos( angles[2] );
	double sy = sin( angles[2] );

	pM->matrix[0] = ( float )( cp*cy );
	pM->matrix[4] = ( float )( cp*sy );
	pM->matrix[8] = ( float )( -sp );

	double srsp = sr*sp;
	double crsp = cr*sp;

	pM->matrix[1] = ( float )( srsp*cy-cr*sy );
	pM->matrix[5] = ( float )( srsp*sy+cr*cy );
	pM->matrix[9] = ( float )( sr*cp );

	pM->matrix[2] = ( float )( crsp*cy+sr*sy );
	pM->matrix[6] = ( float )( crsp*sy-sr*cy );
	pM->matrix[10] = ( float )( cr*cp );
}

void Matrix_SetInverseRotationDegrees(MATRIX * pM, const float *angles )
{
	float vec[3];
	vec[0] = ( float )( angles[0]*180.0/PI );
	vec[1] = ( float )( angles[1]*180.0/PI );
	vec[2] = ( float )( angles[2]*180.0/PI );
	Matrix_SetInverseRotationRadians( pM, vec );
}

void Matrix_SetRotationRadians(MATRIX * pM, const float *angles )
{
	double cr = cos( angles[0] );
	double sr = sin( angles[0] );
	double cp = cos( angles[1] );
	double sp = sin( angles[1] );
	double cy = cos( angles[2] );
	double sy = sin( angles[2] );

	pM->matrix[0] = ( float )( cp*cy );
	pM->matrix[1] = ( float )( cp*sy );
	pM->matrix[2] = ( float )( -sp );

	double srsp = sr*sp;
	double crsp = cr*sp;

	pM->matrix[4] = ( float )( srsp*cy-cr*sy );
	pM->matrix[5] = ( float )( srsp*sy+cr*cy );
	pM->matrix[6] = ( float )( sr*cp );

	pM->matrix[8] = ( float )( crsp*cy+sr*sy );
	pM->matrix[9] = ( float )( crsp*sy-sr*cy );
	pM->matrix[10] = ( float )( cr*cp );
}

void Matrix_Set(MATRIX * pM, const float * pMatrix )
{
	memcpy( pM->matrix, pMatrix, sizeof( float )*16 );
}

void Matrix_LoadIdentity(MATRIX * pM)
{
	memset( pM->matrix, 0, sizeof( float )*16 );
	pM->matrix[0] = pM->matrix[5] = pM->matrix[10] = pM->matrix[15] = 1;
}

void Matrix_RotateVect(MATRIX * pM, float *pVect )
{
	float vec[3];

	vec[0] = pVect[0] * pM->matrix[0] + pVect[1] * pM->matrix[4] + pVect[2] * pM->matrix[8];
	vec[1] = pVect[0] * pM->matrix[1] + pVect[1] * pM->matrix[5] + pVect[2] * pM->matrix[9];
	vec[2] = pVect[0] * pM->matrix[2] + pVect[1] * pM->matrix[6] + pVect[2] * pM->matrix[10];

	memcpy( pVect, vec, sizeof( float )*3 );
}

void Matrix_InverseRotateVect(MATRIX * pM, float *pVect )
{
	float vec[3];

	vec[0] = pVect[0] * pM->matrix[0] + pVect[1] * pM->matrix[1] + pVect[2] * pM->matrix[2];
	vec[1] = pVect[0] * pM->matrix[4] + pVect[1] * pM->matrix[5] + pVect[2] * pM->matrix[6];
	vec[2] = pVect[0] * pM->matrix[8] + pVect[1] * pM->matrix[9] + pVect[2] * pM->matrix[10];

	memcpy( pVect, vec, sizeof( float )*3 );
}

void Matrix_TranslateVect(MATRIX * pM, float *pVect )
{
	pVect[0] += pM->matrix[12];
	pVect[1] += pM->matrix[13];
	pVect[2] += pM->matrix[14];
}

void Matrix_InverseTranslateVect(MATRIX * pM, float *pVect )
{
	pVect[0] -= pM->matrix[12];
	pVect[1] -= pM->matrix[13];
	pVect[2] -= pM->matrix[14];
}

/*
void Matrix_setRotationQuaternion( const Quaternion& quat )
{
	pM->matrix[0] = ( float )( 1.0 - 2.0*quat[1]*quat[1] - 2.0*quat[2]*quat[2] );
	pM->matrix[1] = ( float )( 2.0*quat[0]*quat[1] + 2.0*quat[3]*quat[2] );
	pM->matrix[2] = ( float )( 2.0*quat[0]*quat[2] - 2.0*quat[3]*quat[1] );

	pM->matrix[4] = ( float )( 2.0*quat[0]*quat[1] - 2.0*quat[3]*quat[2] );
	pM->matrix[5] = ( float )( 1.0 - 2.0*quat[0]*quat[0] - 2.0*quat[2]*quat[2] );
	pM->matrix[6] = ( float )( 2.0*quat[1]*quat[2] + 2.0*quat[3]*quat[0] );

	pM->matrix[8] = ( float )( 2.0*quat[0]*quat[2] + 2.0*quat[3]*quat[1] );
	pM->matrix[9] = ( float )( 2.0*quat[1]*quat[2] - 2.0*quat[3]*quat[0] );
	pM->matrix[10] = ( float )( 1.0 - 2.0*quat[0]*quat[0] - 2.0*quat[1]*quat[1] );
}
*/

