// Vector.c

#include <math.h>
#include "Vector.h"
#include "Matrix.h"

void Vector_Reset(VECTOR * pV)
{
	pV->vector[0] = pV->vector[1] = pV->vector[2] = 0.0f;
	pV->vector[3] = 1;
}

void Vector_Set(VECTOR * pV, const float *values )
{
	pV->vector[0] = values[0];
	pV->vector[1] = values[1];
	pV->vector[2] = values[2];
}

void Vector_Add(VECTOR * pV, const VECTOR * pv )
{
	pV->vector[0] += pv->vector[0];
	pV->vector[1] += pv->vector[1];
	pV->vector[2] += pv->vector[2];
	pV->vector[3] += pv->vector[3];
}

void Vector_Normalize(VECTOR * pV)
{
	float len = Vector_Length(pV);

	pV->vector[0] /= len;
	pV->vector[1] /= len;
	pV->vector[2] /= len;
}

float Vector_Length(VECTOR * pV)
{
	return ( float )sqrt( pV->vector[0] * pV->vector[0]+pV->vector[1] * pV->vector[1]+pV->vector[2] * pV->vector[2] );
}


void Vector_Vector(VECTOR * pV, const float *values )
{
	Vector_Set( pV, values );
	pV->vector[3] = 1;
}

void Vector_Transform(VECTOR * pV, const MATRIX * pM )
{
	double vector[4];
	const float *matrix = Matrix_GetMatrix(pM);

	vector[0] = pV->vector[0] * matrix[0]+pV->vector[1] * matrix[4]+pV->vector[2] * matrix[8]+matrix[12];
	vector[1] = pV->vector[0] * matrix[1]+pV->vector[1] * matrix[5]+pV->vector[2] * matrix[9]+matrix[13];
	vector[2] = pV->vector[0] * matrix[2]+pV->vector[1] * matrix[6]+pV->vector[2] * matrix[10]+matrix[14];
	vector[3] = pV->vector[0] * matrix[3]+pV->vector[1] * matrix[7]+pV->vector[2] * matrix[11]+matrix[15];

	pV->vector[0] = ( float )( vector[0] );
	pV->vector[1] = ( float )( vector[1] );
	pV->vector[2] = ( float )( vector[2] );
	pV->vector[3] = ( float )( vector[3] );
}

void Vector_Transform3(VECTOR * pV, const MATRIX * pM )
{
	double vector[3];
	const float *matrix = Matrix_GetMatrix(pM);

	vector[0] = pV->vector[0] * matrix[0]+pV->vector[1] * matrix[4]+pV->vector[2] * matrix[8];
	vector[1] = pV->vector[0] * matrix[1]+pV->vector[1] * matrix[5]+pV->vector[2] * matrix[9];
	vector[2] = pV->vector[0] * matrix[2]+pV->vector[1] * matrix[6]+pV->vector[2] * matrix[10];

	pV->vector[0] = ( float )( vector[0] );
	pV->vector[1] = ( float )( vector[1] );
	pV->vector[2] = ( float )( vector[2] );
	pV->vector[3] = 1.0f;
}


