// demo.c

#include <windows.h>
#include <gl\gl.h>
#include <assert.h>

#include "model.h"
#include "tga.h"
#include "timer.h"

extern BOOL	keys[256];			// Array Used For The Keyboard Routine
extern int	mouse_x, mouse_y;
extern int	mouse_b;

TIMER timer;

// name of animated model (MilkShape ASCII file)
#define		ANIMATION_FILE	"hog.txt" //"character.txt"
#define		RELATIVE_PATH	"data/"		// if no path make ""
// For this example sowtex3.tga is in the data folder
// so we have to adjust the path. See inside the .txt file
// for the name of the texture to load.
static float 	fAngleSpeed = 0.5f;
static float 	fDistSpeed  = 3.5f;
static BOOL		bShowBones  = TRUE;

// animation speed
#define		FRAMES_PER_SECOND	24.0f

typedef struct camera
{	float xangle;
	float yangle;
	float zangle;
	float dist;
}CAMERA;

CAMERA camera = { 10.0f, 90.0f, 0.0f, 400.0f };

void SetProjection(void)
{
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glFrustum(-1.0, 1.0, -1.0, 1.0, 1.0, 1500.0);
}

void UserInput(void)
{
	// Get keyboard and mouse input

	// (clicking in the right half of the window turns the model right
	//  and clicking in the left half turns the model left)
	//
	static int pressed_s = 0;

	if (keys[VK_LEFT] || ((mouse_b & 1)&&mouse_x<320))
		camera.yangle += fAngleSpeed;

	if (keys[VK_RIGHT]|| ((mouse_b & 1)&&mouse_x>320))
		camera.yangle -= fAngleSpeed;

	if (keys[VK_UP]) camera.xangle += fAngleSpeed;

	if (keys[VK_DOWN]) camera.xangle -= fAngleSpeed;


	if (keys[VK_PRIOR]) // page up
		camera.dist -= fDistSpeed;

	if (keys[VK_NEXT]) // page down
		camera.dist += fDistSpeed;

	if (keys['S'] && !pressed_s) // 's' toggle skeleton display
	{
		bShowBones = 1 - bShowBones;
		pressed_s = 1;
	}

	if(!keys['S'])
		pressed_s = 0;
}

void Material_Activate(MATERIAL * pM)
{
	glMaterialfv( GL_FRONT, GL_AMBIENT, pM->ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE, pM->diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR, pM->specular );
	glMaterialfv( GL_FRONT, GL_EMISSION, pM->emissive );
	glMaterialf( GL_FRONT, GL_SHININESS, pM->shininess );

	if ( pM->texture > 0 )
	{
		glBindTexture( GL_TEXTURE_2D, pM->texture );
		glEnable( GL_TEXTURE_2D );
	}
	else
		glDisable( GL_TEXTURE_2D );
}

void Render(MODEL * pM)
{
	int	k, i, j;
	TRI		*pTri  = NULL;
	VEC		*pVec  = NULL;
	NORMAL  *pN    = NULL;
	BONE    *pBone = NULL;
	float	v[3];
	MATRIX	*pMatrix = NULL;

	for(k = 0; k < pM->numMeshes; k++)	// for each shape
	{
		int materialIndex = pM->materialIndices[k];
		if ( materialIndex >= 0 )
		{
			Material_Activate(&pM->pMaterials[materialIndex]);
		}
		else
		{
			// Material properties?
			glDisable( GL_TEXTURE_2D );
		}

		//glEnable(GL_LIGHTING);
		glBegin(GL_TRIANGLES);
		for(i = 0; i < pM->pMeshes[k].numTriangles; i++)	// for each triangle
		{
			pTri = pM->pMeshes[k].triangles + i;				// pointer to triangle

			for(j = 0; j < 3; j++)	// 3 vertices of the triangle
			{

			pN = pM->pMeshes[k].normals + pTri->n[j];
			glNormal3f(pN->x, pN->y, pN->z);	// set normal vector  (object space)

			pVec = pM->pMeshes[k].vertices + pTri->v[j];		// pointer to vertex
			glTexCoord2f(pVec->u, pVec->v);			// texture coordinate

			if( pVec->bone == -1 )
			{
				glVertex3f( pVec->x, pVec->y, pVec->z );
			}
			else
			{
				pBone = &pM->pBones[pVec->bone];
				pMatrix = &pBone->matFinal;
				v[0] = pVec->x;
				v[1] = pVec->y;
				v[2] = pVec->z;

				Matrix_RotateVect(pMatrix, v );
				Matrix_TranslateVect(pMatrix, v );

				glVertex3fv( v );
			}
			}
		}
		glEnd();
	}
}

void Display( MODEL * pModel )
{
    // Clear the RGB buffer and the depth buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // Set the modelview matrix to be the identity matrix
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();

	// Set camera position
   	glTranslatef ( 0.0f, 0.0f, -camera.dist);
	glRotatef (camera.xangle, 1.0f, 0.0f, 0.0f);
	glRotatef (camera.yangle, 0.0f, 1.0f, 0.0f);
	glRotatef (camera.zangle, 0.0f, 0.0f, 1.0f);

    // Translation and rotation of the object goes here

	Render(pModel);				// render model

	if(bShowBones)
	{
		glClear( GL_DEPTH_BUFFER_BIT);	// clear z buffer
		Model_RenderBones(pModel);			// render bones
	}
}

void SetLighting( void )
{
	GLfloat position [] = { 0.3f, 1.0f, 0.3f, 0.0f };

	// Create a Directional Light Source
	glLightfv(GL_LIGHT0, GL_POSITION, position);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

BOOL InitializeModel( MODEL * pModel )
{
	if( !Model_LoadFromMs3dAsciiFile(pModel, RELATIVE_PATH, ANIMATION_FILE ) ){
		MessageBox( 0, "Couldn't load the model", "Error", MB_OK );
		return 0;
	}

	glShadeModel (GL_SMOOTH);

	// Set the polygon mode to fill
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glPolygonMode (GL_BACK, GL_POINTS);

	glEnable (GL_DEPTH_TEST);

	glCullFace (GL_BACK);
	glEnable (GL_CULL_FACE);

	glEnable (GL_TEXTURE_2D);
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	// Set the clear color to black
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	Timer_Timer(&timer); // make a timer
	// disabled for now (makes the model too dark)
	//	SetLighting();

	SetProjection();

	return TRUE;
}


BOOL DrawScene( MODEL * pModel )
{
	static float time;
	float elapsedTime;

	assert( pModel != NULL );

	// get user input
	UserInput();

	// show the model
	Display(pModel);

	// advance the animation in accordance to
	// elapsed time since last call
	elapsedTime = Timer_GetTime(&timer) - time;
	time += elapsedTime;
	Model_AdvanceAnimation(pModel, elapsedTime * FRAMES_PER_SECOND );

	return TRUE;
}
