//  model.h

#ifndef _MODEL_H_
#define _MODEL_H_

#include <stdio.h>
#include <gl\gl.h>
#include "Matrix.h"

// MilkShape constants
#define MS_MAX_NAME   32
#define MS_MAX_PATH   256

#ifndef byte
typedef unsigned char byte;
#endif // byte

typedef struct Vec
{	float	x,y,z;
	float	w;		// 4th homogenous coordinate
	float	u,v;	// texture position
	int		bone;	// index into Model bones array
}VEC;

typedef struct Tri
{	int		v[3];	// indices into vertices[]
	int		n[3];	// indices into normals[], normal per vertex
}TRI;

typedef struct Normal
{	float	x,y,z;
}NORMAL;

typedef struct Mesh
{
	int	   numVertices;
	VEC	  *vertices;

	int	   numTriangles;
	TRI	   *triangles;

	int	   numNormals;
	NORMAL *normals;
}MESH;


typedef struct Material
{
	char  name[MS_MAX_NAME];
	float ambient[4];
	float diffuse[4];
	float specular[4];
	float emissive[4];
	float shininess;
	float transparency;
	char  diffuse_name[MS_MAX_NAME];
	char  alpha_name[MS_MAX_NAME];
	GLuint texture;
}MATERIAL;

typedef struct KeyFrameRot
{	float	time;
	float	position[3];
}KEYFRAMEROT;

typedef struct KeyFramePos
{	float	time;
	float	position[3];
}KEYFRAMEPOS;

typedef struct Bone
{	byte     flags;
	char	 name[MS_MAX_NAME];
	char	 parentName[MS_MAX_NAME];
	float	 rotation[3];
	float	 position[3];
	int		 numPosKeys;
	int		 numRotKeys;
	KEYFRAMEROT *keyFramesRot;
	KEYFRAMEPOS *keyFramesTrans;
	// end original struct
	struct   Bone * parent;			// pointer to parent bone (or NULL)
	MATRIX	 matRelative;				// fixed transformation matrix relative to parent
	MATRIX	 matFinal;					// absolute in accordance to animation
}BONE;

typedef struct Model
{	float	fMaxTime;  		// length of animation (= Frames)
	float	fCurrentTime;	// advancement within animation

	int		numMeshes;
	MESH	*pMeshes;

	int		*materialIndices;

	int		 numMaterials;
	MATERIAL *pMaterials;

	int		numBones;
	BONE	*pBones;
}MODEL;

void Model_Model(MODEL * pM);
void Model_Delete(MODEL * pM);
void Model_RenderBones(MODEL * pM);
BOOL Model_LoadFromMs3dAsciiFile(MODEL * pM, const char *path, const char *filename );
void Model_AdvanceAnimation(MODEL * pModel, float );
void Mesh_Render(MESH * mesh);

#endif

