// TGA.CPP - TGA image loading

#include <stdio.h>
#include "tga.h"

static TextureImage the_texture;

GLubyte uTGAcompare[12] = {0,0,2, 0,0,0,0,0,0,0,0,0};	// Uncompressed TGA Header
GLubyte cTGAcompare[12] = {0,0,10,0,0,0,0,0,0,0,0,0};	// Compressed TGA Header

BOOL TGA_LoadUncompressed(TextureImage * pTexture, GLubyte *header, FILE *file)
{
	GLuint		bytesPerPixel;								// Holds Number Of Bytes Per Pixel Used In The TGA File
	GLuint		imageSize;									// Used To Store The Image Size When Setting Aside Ram
	GLuint		temp;										// Temporary Variable

	pTexture->width  = header[1] * 256 + header[0];			// Determine The TGA Width	(highbyte*256+lowbyte)
	pTexture->height = header[3] * 256 + header[2];			// Determine The TGA Height	(highbyte*256+lowbyte)

 	if(	pTexture->width	<=0	||								// Is The Width Less Than Or Equal To Zero
		pTexture->height	<=0	||								// Is The Height Less Than Or Equal To Zero
		(header[4]!=24 && header[4]!=32))					// Is The TGA 24 or 32 Bit?
	{
		fclose(file);										// If Anything Failed, Close The File
		MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
		return FALSE;
	}

	pTexture->bpp	= header[4];							// Grab The TGA's Bits Per Pixel (24 or 32)
	bytesPerPixel	= pTexture->bpp/8;						// Divide By 8 To Get The Bytes Per Pixel
	imageSize		= pTexture->width*pTexture->height*bytesPerPixel;	// Calculate The Memory Required For The TGA Data

	pTexture->imageData = (GLubyte *)malloc(imageSize);		// Reserve Memory To Hold The TGA Data

	if(	pTexture->imageData == NULL ||							// Does The Storage Memory Exist?
		fread(pTexture->imageData, 1, imageSize, file) != imageSize)	// Does The Image Size Match The Memory Reserved?
	{
		if(pTexture->imageData != NULL)						// Was Image Data Loaded
			free(pTexture->imageData);						// If So, Release The Image Data

		fclose(file);										// Close The File
		MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
		return FALSE;
	}

	for(int i=0; i<(int)imageSize; i+=bytesPerPixel)		// Loop Through The Image Data
	{														// Swaps The 1st And 3rd Bytes ('R'ed and 'B'lue)
		temp=pTexture->imageData[i];							// Temporarily Store The Value At Image Data 'i'
		pTexture->imageData[i] = pTexture->imageData[i + 2];	// Set The 1st Byte To The Value Of The 3rd Byte
		pTexture->imageData[i + 2] = temp;					// Set The 3rd Byte To The Value In 'temp' (1st Byte Value)
	}

	fclose (file);											// Close The File
	return TRUE;
}

BOOL TGA_LoadCompressed(TextureImage * pTexture, GLubyte *header, FILE *file)
{
	GLuint		bytesPerPixel;								// Holds Number Of Bytes Per Pixel Used In The TGA File
	GLuint		imageSize;									// Used To Store The Image Size When Setting Aside Ram

	pTexture->width  = header[1] * 256 + header[0];			// Determine The TGA Width	(highbyte*256+lowbyte)
	pTexture->height = header[3] * 256 + header[2];			// Determine The TGA Height	(highbyte*256+lowbyte)
	pTexture->bpp	= header[4];							// Grab The TGA's Bits Per Pixel (24 or 32)

 	if(	pTexture->width	<=0	||								// Is The Width Less Than Or Equal To Zero
		pTexture->height	<=0	||								// Is The Height Less Than Or Equal To Zero
		(pTexture->bpp !=24 && pTexture->bpp !=32))					// Is The TGA 24 or 32 Bit?
	{
		fclose(file);										// If Anything Failed, Close The File
		MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
		exit (1);
	}

	bytesPerPixel	= pTexture->bpp/8;						// Divide By 8 To Get The Bytes Per Pixel
	imageSize		= pTexture->width*pTexture->height*bytesPerPixel;	// Calculate The Memory Required For The TGA Data

	pTexture->imageData=(GLubyte *)malloc(imageSize);		// Reserve Memory To Hold The TGA Data
	if(	pTexture->imageData==NULL )							// Does The Storage Memory Exist?
	{
		fclose(file);										// Close The File
		MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
		exit (1);
	}


	GLuint pixelcount	= pTexture->height * pTexture->width;							// Nuber of pixels in the image
	GLuint currentpixel	= 0;												// Current pixel being read
	GLuint currentbyte	= 0;												// Current byte
	GLubyte * pColorbuffer = (GLubyte *)malloc(bytesPerPixel);			// Storage for 1 pixel
	do
	{
		GLubyte chunkheader = 0;											// Storage for "chunk" header
		if(fread(&chunkheader, sizeof(GLubyte), 1, file) == 0)				// Read in the 1 byte header
		{
			MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
			if(file != NULL)												// If file is open
			{
				fclose(file);												// Close file
			}
			if(pTexture->imageData != NULL)									// If there is stored image data
			{
				free(pTexture->imageData);									// Delete image data
			}
			return FALSE;													// Return failed
		}
		if(chunkheader < 128)												// If the ehader is < 128, it means the that is the number of RAW color packets minus 1
		{																	// that follow the header
			chunkheader++;													// add 1 to get number of following color values
			for(short counter = 0; counter < chunkheader; counter++)		// Read RAW color values
			{
				if(fread(pColorbuffer, 1, bytesPerPixel, file) != bytesPerPixel) // Try to read 1 pixel
				{
					MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
					if(file != NULL)													// See if file is open
					{
						fclose(file);													// If so, close file
					}
					if(pColorbuffer != NULL)												// See if colorbuffer has data in it
					{
						free(pColorbuffer);												// If so, delete it
					}
					if(pTexture->imageData != NULL)										// See if there is stored Image data
					{
						free(pTexture->imageData);										// If so, delete it too
					}
					return FALSE;														// Return failed
				}
																						// write to memory
				pTexture->imageData[currentbyte		] = pColorbuffer[2];				    // Flip R and B vcolor values around in the process
				pTexture->imageData[currentbyte + 1	] = pColorbuffer[1];
				pTexture->imageData[currentbyte + 2	] = pColorbuffer[0];
				if(bytesPerPixel == 4)												// if its a 32 bpp image
				{
					pTexture->imageData[currentbyte + 3] = pColorbuffer[3];				// copy the 4th byte
				}
				currentbyte += bytesPerPixel;										// Increase thecurrent byte by the number of bytes per pixel
				currentpixel++;															// Increase current pixel by 1
				if(currentpixel > pixelcount)											// Make sure we havent read too many pixels
				{
					MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
					if(file != NULL)													// If there is a file open
					{
						fclose(file);													// Close file
					}
					if(pColorbuffer != NULL)												// If there is data in colorbuffer
					{
						free(pColorbuffer);												// Delete it
					}
					if(pTexture->imageData != NULL)										// If there is Image data
					{
						free(pTexture->imageData);										// delete it
					}
					return FALSE;														// Return failed
				}
			}
		}
		else																			// chunkheader > 128 RLE data, next color reapeated chunkheader - 127 times
		{
			chunkheader -= 127;															// Subteact 127 to get rid of the ID bit
			if(fread(pColorbuffer, 1, bytesPerPixel, file) != bytesPerPixel)		// Attempt to read following color values
			{
				MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
				if(file != NULL)														// If thereis a file open
				{
					fclose(file);														// Close it
				}
				if(pColorbuffer != NULL)													// If there is data in the colorbuffer
				{
					free(pColorbuffer);													// delete it
				}
				if(pTexture->imageData != NULL)											// If thereis image data
				{
					free(pTexture->imageData);											// delete it
				}
				return FALSE;															// return failed
			}
			for(short counter = 0; counter < chunkheader; counter++)					// copy the color into the image data as many times as dictated
			{																			// by the header
				pTexture->imageData[currentbyte		] = pColorbuffer[2];					// switch R and B bytes areound while copying
				pTexture->imageData[currentbyte + 1	] = pColorbuffer[1];
				pTexture->imageData[currentbyte + 2	] = pColorbuffer[0];
				if(bytesPerPixel == 4)												// If TGA images is 32 bpp
				{
					pTexture->imageData[currentbyte + 3] = pColorbuffer[3];				// Copy 4th byte
				}
				currentbyte += bytesPerPixel;										// Increase current byte by the number of bytes per pixel
				currentpixel++;															// Increase pixel count by 1
				if(currentpixel > pixelcount)											// Make sure we havent written too many pixels
				{
					MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
					if(file != NULL)													// If there is a file open
					{
						fclose(file);													// Close file
					}
					if(pColorbuffer != NULL)												// If there is data in colorbuffer
					{
						free(pColorbuffer);												// Delete it
					}
					if(pTexture->imageData != NULL)										// If there is Image data
					{
						free(pTexture->imageData);		// delete it
					}
					return FALSE;						// Return failed
				}
			}
		}
	}
	while(currentpixel < pixelcount);					// Loop while there are still pixels left

	if(pColorbuffer != NULL)							// If there is data in colorbuffer
	{
		free(pColorbuffer);								// Delete it
	}

	fclose (file);										// Close The File
	return TRUE;										// return success
}

int TGA_LoadTexture(char *filename,  GLuint *pTex)			// Loads A TGA File Into Memory
{
	GLubyte		TGAcompare[12];								// Used To Compare TGA Header
	GLubyte		header[6];									// First 6 Useful Bytes From The Header
	GLuint		type = GL_RGBA;								// Set The Default GL Mode To RBGA (32 BPP)
	TextureImage * pTexture;
	FILE *file;

	file = fopen(filename, "rb");						// Open The TGA File

	pTexture = &the_texture;

	if(	file==NULL )										// Does File Even Exist?
	{
		MessageBox(0, "Cannot open texture file", "Error", MB_OK);
		exit (1);
	}

	if(	fread(TGAcompare, 1, sizeof(TGAcompare), file) != sizeof(TGAcompare) )	// Are There 12 Bytes To Read?
	{
		fclose(file);										// If Anything Failed, Close The File
		MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
		exit (1);
	}

	if(	fread(header, 1, sizeof(header), file) != sizeof(header))	// If So Read Next 6 Header Bytes
	{
		fclose(file);										// If Anything Failed, Close The File
		MessageBox(0, "Error loading texture bitmap", "Error", MB_OK);
		exit (1);
	}

	if(memcmp(uTGAcompare, &TGAcompare, sizeof(TGAcompare)) == 0)		// See if header matches the predefined header of
	{																	// an Uncompressed TGA image
		TGA_LoadUncompressed(pTexture, header, file);					// If so, jump to Uncompressed TGA loading code
	}
	else if(memcmp(cTGAcompare, &TGAcompare, sizeof(TGAcompare)) == 0)	// See if header matches the predefined header of
	{																	// an RLE compressed TGA image
		TGA_LoadCompressed(pTexture, header, file);					  // If so, jump to Compressed TGA loading code
	}

	// Build A Texture From The Data
	glGenTextures(1, &pTexture->texID);					// Generate OpenGL texture IDs

	glBindTexture(GL_TEXTURE_2D, pTexture->texID);						// bind the texture
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// linear filtered
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// linear filtered

	if (pTexture->bpp == 24)						// Was The TGA 24 Bits
	{
		type = GL_RGB;							// If So Set The 'type' To GL_RGB
	}

	glTexImage2D(GL_TEXTURE_2D, 0, type, pTexture->width, pTexture->height, 0, type, GL_UNSIGNED_BYTE, pTexture->imageData);

	*pTex = pTexture->texID;

	// not needed
	if(pTexture->imageData) free(pTexture->imageData);

	return 1;									// Texture Building Went Ok, Return True
}
