// timer.c
//
// Windows timer functions
// See NeHe lesson 21
//
// Timer functions encapsulated in simple Timer class
// value returns in seconds, not milliseconds
//

#include "timer.h"

void Timer_Timer(TIMER * pT)						// Initialize Our Timer (Get It Ready)
{
	memset(pT, 0, sizeof(TIMER));					// Clear Our Timer Structure

	// Check To See If A Performance Counter Is Available
	// If One Is Available The Timer Frequency Will Be Updated
	if (!QueryPerformanceFrequency((LARGE_INTEGER *) &pT->frequency))
	{
		// No Performace Counter Available
		pT->performance_timer	= FALSE;				// Set Performance Timer To FALSE
		pT->mm_timer_start		= timeGetTime();		// Use timeGetTime() To Get Current Time
		pT->resolution			= 1.0f/1000.0f;			// Set Our Timer Resolution To .001f
		pT->frequency			= 1000;					// Set Our Timer Frequency To 1000
		pT->mm_timer_elapsed	= pT->mm_timer_start;	// Set The Elapsed Time To The Current Time
	}
 	else
	{
		// Performance Counter Is Available, Use It Instead Of The Multimedia Timer
		// Get The Current Time And Store It In performance_timer_start
		QueryPerformanceCounter((LARGE_INTEGER *) &pT->performance_timer_start);
		pT->performance_timer		= TRUE;				// Set Performance Timer To TRUE
		// Calculate The Timer Resolution Using The Timer Frequency
		pT->resolution		= (float) (((double)1.0f)/((double)pT->frequency));
		// Set The Elapsed Time To The Current Time
		pT->performance_timer_elapsed	= pT->performance_timer_start;
	}
}

// Note: in contrast to NeHe's version this returns an elapsed time in seconds, not ms!
float Timer_GetTime(TIMER * pT)								// Get Time In Seconds
{
	float f;
	long long time;											// time Will Hold A 64 Bit Integer
	DWORD dw;
	if (pT->performance_timer)								// Are We Using The Performance Timer?
	{
		QueryPerformanceCounter((LARGE_INTEGER *) &time);	// Grab The Current Performance Time

		// Return The Current Time Minus The Start Time Multiplied By The Resolution
		return ( (float) ( time - pT->performance_timer_start) * pT->resolution);
	}
	else
	{
		// Return The Current Time Minus The Start Time Multiplied By The Resolution
		dw = timeGetTime();
	   dw = pT->mm_timer_start;
	   dw = pT->resolution;
		f  = (float) ( timeGetTime() - pT->mm_timer_start) * pT->resolution;
		return( (float) ( timeGetTime() - pT->mm_timer_start) * pT->resolution);
	}
}


